% This is the main script to run the empirical application
clear 
clc
warning off

%% Load data 
% from 2003 January to 2016 January

[cds,cdslabel]              = xlsread('CDS.csv');
[lstock,lstocklabel]        = xlsread('LocalStock.csv');
[fxreserves,fxreslabel]     = xlsread('FXReserves.csv');
[fxrates,fxratelabel]       = xlsread('FXRates.csv');
[globaldata,glabel]         = xlsread('Global.csv');

%% Data transformation
cdschange           = zeros(size(cds,1)-1,size(cds,2));                             % change of CDS spread
for i = 1:length(cds)-1
    cdschange(i,:)  = cds(i+1,:)-cds(i,:);
end
fxreserves          = fxreserves*100;                                               % percentage of foreigen exchange reserves
fxrates             = fxrates*100;                                                  % percentage of foreigen exchange rates
N                   = size(cdschange,2);
T                   = size(cdschange,1);
for i = 1:N
    x(i,:,:)        = [lstock(2:end,i) fxrates(2:end,i) fxreserves(2:end,i) globaldata(2:end,:)];
    y(i,:)          = cdschange(:,i);
end
K                   = size(x,3);
for i = 1:N
    for kk = 1:K
        x(i,:,kk)   = (x(i,:,kk)-mean(x(i,:,kk)))./std(x(i,:,kk));
        y(i,:)      = (y(i,:)-mean(y(i,:)))./std(y(i,:));
    end
end

%% Determining the number of breaks
maxB                        = 8;                                            % maximum number of breaks
TSSR                        = zeros(maxB,1);
for i = 1:N
    gx                      = permute(x(i,:,:),[2,3,1]);    
    gy                      = y(i,:)';    
    e                       = gy-gx*((gx'*gx)\(gx'*gy));            
    ssr(i)                  = e'*e;        
end
TSSR(1)                     = sum(ssr) + TSSR(1);
for B = 2:maxB
    clear regime    
    % detect break points
    ry                 = y';
    for kk = 1:K
        rx(:,:,kk)     = x(:,:,kk)';
    end
    hatb               = binary_seg(ry,rx,N,T,K);
    sidx               = 1:T;
    regime(sidx(1:hatb),1)      = 1;                                             
    regime(sidx(hatb+1:end),2)  = 1;                                             
    SSR                         = realmax*ones(B,1);
    if B == 2
        regime                      = regime;
    else
        bb                          = 3;
        while bb <= B
            for bk  = 1:bb-1
                sample              = regime(:,bk) == 1;                     
                sampleidx           = find(sample);                              
                Tg                  = length(sampleidx);
                if Tg == 1
                    ssr_tmp         = realmax;
                else
                    subsamp_rx      = rx(sample,:,:);                            
                    subsamp_ry      = ry(sample,:);                              
                    hatb            = binary_seg(subsamp_ry,subsamp_rx,N,Tg,K);
                    reg_mem_tmp     = regime;
                    reg_mem_tmp(sampleidx(hatb+1:end),1:(bb-1))  = 0;           % further division of subsample
                    reg_mem_tmp(sampleidx(hatb+1:end),bb)        = 1;
                    ssr_tmp         = ssr_break(ry,rx,reg_mem_tmp);             % ssr of the focused regime after further segmentation                        
                    if ssr_tmp < SSR(bb)                                        % choose division with smaller ssr
                        SSR(bb)        = ssr_tmp;
                        reg_mem_best   = reg_mem_tmp;
                    end
                end
            end
            regime                  = reg_mem_best;
            bb                      = bb + 1;
        end
    end
    % sum of squared residuals
    NN                  = 1:T;        
    gi                  = logical(regime);    
    for bb = 1:B
        clear e
        this_group          = gi(:,bb);        
        g_index             = NN(this_group);
        for i = 1:N
            gx              = permute(x(i,g_index,:),[2,3,1]);
            gy              = y(i,g_index)';
            e               = gy-gx*((gx'*gx)\(gx'*gy));
            ssr(i)          = e'*e;
        end
        TSSR(B)             = sum(ssr) + TSSR(B);
    end
end
numbreak                    = 1:maxB;
BIC                         = TSSR'/(N*T) + TSSR(maxB)/(N*T)*size(x,3)*numbreak*N*log(N*T)/(N*T); 

%% Break detection based on selected number of breaks
[~,B]              = min(BIC);                                              % number of breaks selected by the BIC
ry                 = y';
for kk = 1:K
    rx(:,:,kk)     = x(:,:,kk)';
end
hatb               = binary_seg(ry,rx,N,T,K);
sidx               = 1:T;
regime(sidx(1:hatb),1)      = 1;                                             
regime(sidx(hatb+1:end),2)  = 1;                                             
SSR                         = realmax*ones(B,1);
if B == 2
    return;
else
    bb                          = 3;
    while bb <= B
        for bk  = 1:bb-1
            sample              = regime(:,bk) == 1;                     
            sampleidx           = find(sample);                              
            Tg                  = length(sampleidx);
            if Tg == 1
                ssr_tmp         = realmax;
            else
                subsamp_rx      = rx(sample,:,:);                            
                subsamp_ry      = ry(sample,:);                             
                hatb            = binary_seg(subsamp_ry,subsamp_rx,N,Tg,K);
                reg_mem_tmp     = regime;
                reg_mem_tmp(sampleidx(hatb+1:end),1:(bb-1))  = 0;            
                reg_mem_tmp(sampleidx(hatb+1:end),bb)        = 1;
                ssr_tmp         = ssr_break(ry,rx,reg_mem_tmp);                            
                if ssr_tmp < SSR(bb)                                           
                    SSR(bb)         = ssr_tmp;
                    reg_mem_best   = reg_mem_tmp;
                end
            end
        end
        regime                  = reg_mem_best;
        bb                      = bb + 1;
    end
end
for bb = 1:B
    [est_regime(bb)]  = min(find(regime(:,bb)));                            % estimated break points
end

%% coefficient estimation without bootstrap
startpt             = 1;                                                    % using the observations in the first regime (t=1, ... T_1-1)
%startpt             = est_regime(end-1);                                    % using the observations in the second regime (t = T_1,...T_2-1)
%startpt             = est_regime(end);                                      % using the observations in the third regime (t = T_2,...T)
xpred               = x(:,1:est_regime(end-1)-1,:);
ypred               = y(:,2:est_regime(end-1),:);
Tpred               = length(1:est_regime(end-1)-1);
lambda_tmp          = 2^(-4);
pred                = 0;
boot                = 0;
outsample_ratio     = 0;
[b_mpa,~]           = mpa_est(xpred,ypred,N,Tpred,K,pred,boot,lambda_tmp,outsample_ratio);   
b_mpa               = reshape(b_mpa,K,N);

